@@ -2,11 +2,11 @@
 
 # Copyright (C) 2009 Roderick B. Greening <roderick.greening@gmail.com>
 # Copyright (C) 2014 Harald Sitter <apachelogger@kubuntu.org>
-# 
+#
 # Based in part on work by:
 #  David Edmundson <kde@davidedmundson.co.uk>
 #  Canonical Ltd. USB Creator Team
-# 
+#
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3,
 # as published by the Free Software Foundation.
@@ -19,15 +19,15 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+# TODO: When pykf5 becomes available qmessagebox should be ported back to kmessagebox
+
 import sys
 import os
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
-from PyQt4 import uic
-from PyKDE4.kdeui import KIcon, KMessageBox, KGlobalSettings
-from PyKDE4.kdecore import KProcess, KStandardDirs, KUrl, i18n
-from PyKDE4.kio import KFileDialog
+from PyQt5 import uic
+from PyQt5.QtCore import *
+from PyQt5.QtGui import *
+from PyQt5.QtWidgets import *
 
 from usbcreator.frontends.kde.translate import translate
 uic.properties.Properties._string = translate
@@ -51,19 +51,20 @@ def thread_wrap(func):
 class KdeFrontend(Frontend, QObject):
     @classmethod
     def startup_failure(cls, message):
-        KMessageBox.sorry(None, message, "", KMessageBox.Notify)
+        #KMessageBox.error(None, message)
+        QMessageBox.critical(None, '', message)
+        sys.exit(1)
 
     @classmethod
     def DBusMainLoop(cls):
         from dbus.mainloop.qt import DBusQtMainLoop
         DBusQtMainLoop(set_as_default=True)
 
-    def __init__(self, backend, img=None, persistent=True,
+    def __init__(self, backend, img=None,
                  allow_system_internal=False):
         QObject.__init__(self)
 
         #our passed vars - keep them private
-        self.__persistent = persistent
         self.__img = img
         self.__allow_system_internal = allow_system_internal
 
@@ -85,14 +86,12 @@ class KdeFrontend(Frontend, QObject):
         self.__backend.install_progress_message_cb = self.progress_message
         self.__backend.retry_cb = self.retry
         self.__backend.target_changed_cb = self.update_target
-        self.__backend.format_ended_cb = self.format_ended
-        self.__backend.format_failed_cb = self.format_failed
 
         #add any file sources passed
         if self.__img is not None:
             self.__backend.add_image(misc.text_type(self.__img))
 
-        downloadsDir = QDir(KGlobalSettings.self().downloadPath())
+        downloadsDir = QDir(QStandardPaths.standardLocations(QStandardPaths.DownloadLocation)[0])
         isoFilter = []
         isoFilter.append("*.iso")
         for iso in downloadsDir.entryList(isoFilter, QDir.Files):
@@ -133,56 +132,33 @@ class KdeFrontend(Frontend, QObject):
         if 'USBCREATOR_LOCAL' in os.environ:
             appdir = os.path.join(os.getcwd(), 'gui')
         else:
-            file = KStandardDirs.locate("appdata", self.__mainWindow_ui)
+            file = QStandardPaths.locate(QStandardPaths.DataLocation, self.__mainWindow_ui)
             appdir = file[:file.rfind("/")]
         uic.loadUi(misc.text_type(appdir + "/" + self.__mainWindow_ui), self.__mainWindow)
 
-        #set default persist size
-        self.__mainWindow.ui_persist_label.setText(misc.format_mb_size(128))
-
-        #set persistent ui elements state
-        if self.__persistent:
-            self.__mainWindow.ui_persist_enabled.setChecked(True)
-            self.__mainWindow.ui_persist_text.setEnabled(True)
-            self.__mainWindow.ui_persist_slider.setEnabled(True)
-            self.__mainWindow.ui_persist_label.setEnabled(True)
-        else:
-            self.__mainWindow.ui_persist_disabled.setChecked(True)
-            self.__mainWindow.ui_persist_frame.hide()
-
         #hide sources if an argument was provided
-        if self.__img is not None:
+        if self.__img is not '':
             self.__mainWindow.ui_source_list.hide()
             self.__mainWindow.ui_add_source.hide()
             self.__mainWindow.source_label.hide()
 
-        #disable the start button and persist frame by default
-        self.__mainWindow.ui_persist_frame.setEnabled(False)
+        #disable the start button by default
         self.__mainWindow.ui_start_button.setEnabled(False)
 
         #add some buttons
-        self.__mainWindow.ui_quit_button.setIcon(KIcon("application-exit"))
-        self.__mainWindow.ui_start_button.setIcon(KIcon("dialog-ok-apply"))
-        self.__mainWindow.ui_add_source.setIcon(KIcon("media-optical"))
-        self.__mainWindow.ui_format_dest.setIcon(KIcon("drive-removable-media-usb-pendrive"))
+        self.__mainWindow.ui_quit_button.setIcon(QIcon.fromTheme("application-exit"))
+        self.__mainWindow.ui_start_button.setIcon(QIcon.fromTheme("dialog-ok-apply"))
+        self.__mainWindow.ui_add_source.setIcon(QIcon.fromTheme("media-optical"))
 
         #set up signals
-        self.connect(self.__mainWindow.ui_add_source,SIGNAL('clicked()'),
-            self.add_file_source_dialog)
-        self.connect(self.__mainWindow.ui_persist_slider,SIGNAL('valueChanged(int)'),
-            lambda value: self.__mainWindow.ui_persist_label.setText(misc.format_mb_size(value)))
-        self.connect(self.__mainWindow.ui_quit_button,SIGNAL('clicked()'),
-            self.quit)
-        self.connect(self.__mainWindow.ui_start_button,SIGNAL('clicked()'),
-            self.install)
-        self.connect(self.__mainWindow.ui_dest_list,SIGNAL(
-            'currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)'),
-            self.dest_selection_changed)
-        self.connect(self.__mainWindow.ui_source_list,SIGNAL(
-            'currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)'),
-            self.source_selection_changed)
-        self.connect(self.__mainWindow.ui_format_dest,SIGNAL('clicked()'),
-            self.format_dest_clicked)
+        self.__mainWindow.ui_add_source.clicked.connect(self.add_file_source_dialog)
+        self.__mainWindow.ui_quit_button.clicked.connect(self.quit)
+        self.__mainWindow.ui_start_button.clicked.connect(self.install)
+        self.__mainWindow.ui_dest_list.currentItemChanged.connect(self.dest_selection_changed)
+        self.__mainWindow.ui_source_list.currentItemChanged.connect(self.source_selection_changed)
+
+        # FIXME: we need a custom delegate and elide the iso column on the left rather than the right
+        #        otherwise long paths will take up the entire space while in fact the image name is the useful bit of information 90% of the time
 
         self.__mainWindow.ui_source_list.setSortingEnabled(True)
         self.__mainWindow.ui_source_list.sortByColumn(0, Qt.AscendingOrder)
@@ -194,17 +170,16 @@ class KdeFrontend(Frontend, QObject):
         # usage.
         # Neither colum is resizable by the user, so additional tooltips are
         # enabled for the widgetitems (see add_source).
-        self.__mainWindow.ui_source_list.header().setResizeMode(0, QHeaderView.Stretch)
-        self.__mainWindow.ui_source_list.header().setResizeMode(1, QHeaderView.Stretch)
-        self.__mainWindow.ui_source_list.header().setResizeMode(2, QHeaderView.ResizeToContents)
+        self.__mainWindow.ui_source_list.header().setSectionResizeMode(0, QHeaderView.Stretch)
+        self.__mainWindow.ui_source_list.header().setSectionResizeMode(1, QHeaderView.Stretch)
+        self.__mainWindow.ui_source_list.header().setSectionResizeMode(2, QHeaderView.ResizeToContents)
         self.__mainWindow.ui_dest_list.setSortingEnabled(True)
         self.__mainWindow.ui_dest_list.sortByColumn(0, Qt.AscendingOrder)
         # For destinations only stretch the device column.
-        self.__mainWindow.ui_dest_list.header().setResizeMode(0, QHeaderView.Stretch)
-        self.__mainWindow.ui_dest_list.header().setResizeMode(1, QHeaderView.ResizeToContents)
-        self.__mainWindow.ui_dest_list.header().setResizeMode(2, QHeaderView.ResizeToContents)
-        self.__mainWindow.ui_dest_list.header().setResizeMode(3, QHeaderView.ResizeToContents)
-        self.progress_bar = QProgressDialog("",i18n('Cancel'),0,100,self.__mainWindow)
+        self.__mainWindow.ui_dest_list.header().setSectionResizeMode(0, QHeaderView.Stretch)
+        self.__mainWindow.ui_dest_list.header().setSectionResizeMode(1, QHeaderView.ResizeToContents)
+        self.__mainWindow.ui_dest_list.header().setSectionResizeMode(2, QHeaderView.ResizeToContents)
+        self.progress_bar = QProgressDialog("",_('Cancel'),0,100,self.__mainWindow)
         #set title of progress window (same as gtk frontend)
         self.progress_bar.setWindowTitle(_('Installing'))
         #prevent progress bar from emitting reset on reaching max value (and auto closing)
@@ -212,9 +187,9 @@ class KdeFrontend(Frontend, QObject):
         #force immediate showing, rather than waiting...
         self.progress_bar.setMinimumDuration(0)
         #must disconnect the canceled() SIGNAL, otherwise the progress bar is actually destroyed
-        self.disconnect(self.progress_bar,SIGNAL('canceled()'),self.progress_bar.cancel)
+        self.progress_bar.canceled.disconnect(self.progress_bar.cancel)
         #now we connect our own signal to display a warning dialog instead
-        self.connect(self.progress_bar,SIGNAL('canceled()'),self.warning_dialog)
+        self.progress_bar.canceled.connect(self.warning_dialog)
 
         #show the window
         self.__mainWindow.show()
@@ -231,11 +206,11 @@ class KdeFrontend(Frontend, QObject):
         '''Handle Failed Install Gracefully'''
 
         logging.exception('Installation failed.')
-        self.__backend.unmount()
         self.progress_bar.hide()
         if not message:
             message = _('Installation failed.')
-        KMessageBox.error(self.__mainWindow, message)
+        #KMessageBox.error(self.__mainWindow, message)
+        QMessageBox.critical(self.__mainWindow, '', message)
         sys.exit(1)
 
     def add_timeout(self, interval, func, *args):
@@ -244,9 +219,7 @@ class KdeFrontend(Frontend, QObject):
 
         # FIXME: now that we are part of a Qt object, we may be able to alter for builtin timers
         timer = QTimer()
-        QObject.connect(timer,
-            SIGNAL('timeout()'),
-            lambda: self.__timeout_callback(func, *args))
+        timer.timeout.connect(lambda: self.__timeout_callback(func, *args))
         timer.start(interval)
 
         return timer
@@ -267,7 +240,7 @@ class KdeFrontend(Frontend, QObject):
         # the new_item lines should be auto triggered onChange to the
         # TreeWidget when new_item is appended.
         new_item.setText(0,target)
-        new_item.setIcon(0,KIcon("drive-removable-media-usb-pendrive"))
+        new_item.setIcon(0,QIcon.fromTheme("drive-removable-media-usb-pendrive"))
 
         item = self.__mainWindow.ui_dest_list.currentItem()
         if not item:
@@ -285,12 +258,6 @@ class KdeFrontend(Frontend, QObject):
             new_item.setToolTip(1, new_item.text(1))
             new_item.setText(2,misc.format_size(dev['capacity']))
             new_item.setToolTip(2, new_item.text(2))
-            free = dev['free']
-            if free >= 0:
-                new_item.setText(3,misc.format_size(free))
-            else:
-                new_item.setText(3,'')
-            new_item.setToolTip(3, new_item.text(3))
 
     def remove_target(self, target):
         for i in range(0,self.__mainWindow.ui_dest_list.topLevelItemCount()):
@@ -312,7 +279,7 @@ class KdeFrontend(Frontend, QObject):
         # the new_item lines should be auto triggered onChange to the TreeWidget
         # when new_item is appended.
         new_item.setText(0,source)
-        new_item.setIcon(0,KIcon("media-optical"))
+        new_item.setIcon(0,QIcon.fromTheme("media-optical"))
 
         item = self.__mainWindow.ui_source_list.currentItem()
         if not item:
@@ -372,30 +339,17 @@ class KdeFrontend(Frontend, QObject):
             logging.debug('No target selected.')
             return ''
 
-    def get_persistence(self):
-        if (self.__mainWindow.ui_persist_enabled.isChecked() and
-            self.__mainWindow.ui_persist_frame.isEnabled()):
-            val = self.__mainWindow.ui_persist_slider.value()
-            return int(val)
-        else:
-            return 0
-
     def update_target(self, udi):
         for i in range(0,self.__mainWindow.ui_dest_list.topLevelItemCount()):
             item = self.__mainWindow.ui_dest_list.topLevelItem(i)
             if misc.text_type(item.data(0,Qt.UserRole)) == udi:
-                self.__mainWindow.ui_dest_list.emit(
-                    SIGNAL('itemChanged(item,0)'))
+                # FIXME: pyqt5 entirely bypasses qt's signals and apparently fails
+                #        to do so correctly so the following yields an error
+                #        even though it should work just fine (i.e. the signal exists
+                #        on the cpp side...
+                self.__mainWindow.ui_dest_list.itemChanged.emit(item, 0)
                 break
-        # Update persistence maximum value.
-        self.__mainWindow.ui_persist_frame.setEnabled(False)
-        self.__mainWindow.ui_start_button.setEnabled(False)
         target = self.__backend.targets[udi]
-        persist_mb = target['persist'] / 1024 / 1024
-        if persist_mb > misc.MIN_PERSISTENCE:
-            self.__mainWindow.ui_persist_frame.setEnabled(True)
-            self.__mainWindow.ui_persist_slider.setRange(
-                misc.MIN_PERSISTENCE, persist_mb)
         # Update install button state.
         status = target['status']
         source = self.__backend.get_current_source()
@@ -407,15 +361,9 @@ class KdeFrontend(Frontend, QObject):
             self.__mainWindow.ui_start_button.setEnabled(True)
         else:
             self.__mainWindow.ui_start_button.setEnabled(False)
-            status = misc.NEED_FORMAT
         # Update the destination status message.
         if status == misc.CANNOT_USE:
             msg = _('The device is not large enough to hold this image.')
-        elif status == misc.NEED_SPACE:
-            msg = _('There is not enough free space for this image.')
-            # FIXME RBG: 04/01/10 - Need Open button for folder to device for removing files
-        elif status == misc.NEED_FORMAT:
-            msg = _('The device needs to be formatted for use.')
         else:
             msg = ''
         self.__mainWindow.ui_dest_status.setText(msg)
@@ -433,10 +381,6 @@ class KdeFrontend(Frontend, QObject):
         self.dest_selection_changed(item, None)
 
     def dest_selection_changed(self, current_item, prev_item):
-        '''The selected partition has changed and the bounds on the persistence
-        slider need to be changed, or the slider needs to be disabled, to
-        reflect the amount of free space on the partition.'''
-
         if not self.__backend:
             return
 
@@ -449,14 +393,19 @@ class KdeFrontend(Frontend, QObject):
     def add_file_source_dialog(self):
         self.__recently_added_image = None
         filename = ''
-        filter = '*.iso|' + _('CD Images') + '\n*.img|' + _('Disk Images')
+        # This here filter is for kfiledialog, no clue if it will ever make a return
+        #filter = '*.iso|' + _('CD Images') + '\n*.img|' + _('Disk Images')
+        filter = _('CD Images') + '(*.iso)' + ';;' + _('Disk Images') + '(*.img)'
+
+        downloadPath = QStandardPaths.standardLocations(QStandardPaths.DownloadLocation)[0]
+        openFileName = QFileDialog.getOpenFileName(self.__mainWindow,
+                                                   '',
+                                                   downloadPath,
+                                                   filter)
+        if openFileName:
+            openFileName = openFileName[0]
+        filename = misc.text_type(openFileName)
 
-        # FIXME: should set the default path KUrl to users home dir...
-        # This is all screwy as its run as root under kdesudo... Home = root and not user.. blarg!
-        # Need to convert to plain string for backend to work
-        filename = misc.text_type(KFileDialog.getOpenFileName(KUrl(KGlobalSettings.self().downloadPath()),
-                                                              filter,
-                                                              self.__mainWindow))
         if not filename:
             return
 
@@ -473,15 +422,23 @@ class KdeFrontend(Frontend, QObject):
     def install(self):
         source = self.get_source()
         target = self.get_target()
-        persist = self.get_persistence()
         if (source and target):
+            msgbox = QMessageBox(self.__mainWindow)
+            msgbox.setIcon(QMessageBox.Warning)
+            msgbox.setText(_('Are you sure you want to write the disc image to the device?'))
+            msgbox.setInformativeText(_('All existing data will be lost.'))
+            msgbox.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
+            res = msgbox.exec_()
+            if res != QMessageBox.Yes:
+                return
+
             self.__mainWindow.hide()
             self.delete_timeout(self.update_loop)
             starting_up = _('Starting up')
             self.progress_bar.setLabelText(starting_up)
             self.progress_bar.show()
             try:
-                self.__backend.install(source, target, persist,
+                self.__backend.install(source, target,
                                        allow_system_internal=self.__allow_system_internal)
             except:
                 self.__fail()
@@ -490,10 +447,14 @@ class KdeFrontend(Frontend, QObject):
             self.notify(message)
 
     @thread_wrap
-    def progress(self, complete, remaining, speed):
+    def progress(self, complete):
         # Updating value cause dialog to re-appear from hidden (dunno why)
         if not self.progress_bar.isHidden():
+            if complete > 100:
+                complete = 100
             self.progress_bar.setValue(int(complete))
+            if complete == 100:
+                self.progress_bar.setLabelText(_('Finishing...'))
 
     @thread_wrap
     def progress_message(self, message):
@@ -511,13 +472,13 @@ class KdeFrontend(Frontend, QObject):
     @thread_wrap
     def success(self):
         '''Install completed'''
-        self.__backend.unmount()
         self.progress_bar.hide()
         text = _('The installation is complete.  You may now reboot your '
                  'computer with this device inserted to try or install '
                  'Ubuntu.')
 
-        KMessageBox.information(self.__mainWindow, text)
+        QMessageBox.information(self.__mainWindow, '', text)
+        self.__backend.shutdown()
         sys.exit(0)
 
     @thread_wrap
@@ -526,12 +487,16 @@ class KdeFrontend(Frontend, QObject):
 
         caption = _('Retry?')
 
-        res = KMessageBox.warningYesNo(self.__mainWindow,message,caption)
+        #res = KMessageBox.warningYesNo(self.__mainWindow,message,caption)
+        res = QMessageBox.warning(self.__mainWindow, caption, message,
+                                  QMessageBox.Yes, QMessageBox.No)
 
-        return res == KMessageBox.Yes
+        #return res == KMessageBox.Yes
+        return res == QMessageBox.Yes
 
     def notify(self,title):
-        KMessageBox.sorry(self.__mainWindow,title)
+        #KMessageBox.sorry(self.__mainWindow,title)
+        QMessageBox.warning(self.__mainWindow, '', title)
 
     def warning_dialog(self):
         '''A warning dialog to show when progress dialog cancel is pressed'''
@@ -542,39 +507,17 @@ class KdeFrontend(Frontend, QObject):
         #hide the progress bar - install will still continue in bg
         self.progress_bar.hide()
 
-        res = KMessageBox.warningYesNo(self.__mainWindow,text,caption)
+        #res = KMessageBox.warningYesNo(self.__mainWindow,text,caption)
+        res = QMessageBox.warning(self.__mainWindow, caption, text,
+                                  QMessageBox.Yes, QMessageBox.No)
 
-        if res == KMessageBox.Yes:
+        #if res == KMessageBox.Yes:
+        if res == QMessageBox.Yes:
             self.quit()
 
         #user chose not to quit, so re-show progress bar
         self.progress_bar.show()
 
-    def format_ended(self):
-        self.__mainWindow.unsetCursor()
-        self.__mainWindow.ui_format_dest.setEnabled(True)
-
-    @thread_wrap
-    def format_failed(self, message):
-        self.__mainWindow.unsetCursor()
-        self.__mainWindow.ui_format_dest.setEnabled(True)
-        # TODO sort through error types (message.get_dbus_name()) in backend,
-        # individual functions in frontend for each error type.
-        KMessageBox.error(self.__mainWindow, str(message))
-
     def format_dest_clicked(self):
-        item = self.__mainWindow.ui_dest_list.currentItem()
-        if not item:
-            return
-        udi = misc.text_type(item.data(0,Qt.UserRole))
-
-        text = _('Are you sure you want to erase the entire disk?')
-
-        res = KMessageBox.warningYesNo(self.__mainWindow,text)
-
-        if res == KMessageBox.Yes:
-            self.__mainWindow.setCursor(Qt.BusyCursor)
-            # Disable simultaneous format attempts.. let current finish first
-            self.__mainWindow.ui_format_dest.setEnabled(False)
-            self.__backend.format(udi)
+        return
 
